==> tinyerp-3.0.tar.gz: 3.0/server/bin/addons/__init__.py <==
##############################################################################
#
# Copyright (c) 2004 TINY SPRL. (http://tiny.be)
#
# $Id$
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contact a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import os, sys, imp

import tools

import netsvc
logger = netsvc.Logger()

opj = os.path.join
ad = tools.config['addons_path']
sys.path.insert(1,ad)

#FIXME: c est pas une bonne solution car ca pete les 'bookmarks' 
# en fait, ce qu'il faudrait c'est de creer une liste des menus qui sont updates pdt l'update all, et virer ceux qui ne sont
# pas dans cette liste apres l'update all... pour bien faire, faudrait pouvoir faire ca aussi module par module mais dans l'etat actuel des choses,
# c'est pas possible vu qu'on a pas l'information necessaire 

#if tools.config["update"].has_key("all"):
#	cr.execute("delete from ir_ui_menu where id != 1")
#	cr.commit()

# create the list of modules to load
loaded={}
unloaded={}
init={}
update={}
#CHECKME: je vois pas l'interet de dico 'active' vu qu'on vire de ttes facons les non actifs juste apres...
active={}
for i in os.listdir(ad):
	if os.path.isfile(opj(ad, i, "__terp__.py")):
		info = eval(file(opj(ad, i, "__terp__.py")).read())
		if info.get('active', True):
			unloaded[i] = info['depends']
			init[i] = info.get('init_xml', [])
			update[i] = info.get('update_xml', [])
		active[i]=info.get('active',True)

# load modules
while len(unloaded):
	for (m,dep) in unloaded.items():
		ready = True
		for j in dep:
			if not (j in loaded or j in unloaded):
				logger.notifyChannel("init", netsvc.LOG_ERROR, "addon:%s:couldn't load module because of unmet dependency (%s) !" % (m,j))
				del unloaded[m]
			ready = ready and j in loaded
		if not active[m]:
			del unloaded[m]
			continue
		if ready:
			logger.notifyChannel("init", netsvc.LOG_INFO, 'addon:%s' % (m,))
			sys.stdout.flush()
			imp.load_module(m, *imp.find_module(m))
			idref = {}
			if tools.config["init"].has_key(m) or tools.config["init"].has_key("all"):
				for xml in init.get(m,[]):
					logger.notifyChannel("init", netsvc.LOG_INFO, 'addon:%s:init %s' % (m,xml))
					tools.convert_xml_import(tools.file_open(opj(m,xml)).read(), idref)
				for xml in update.get(m,[]):
					logger.notifyChannel("init", netsvc.LOG_INFO, 'addon:%s:init %s' % (m,xml))
					tools.convert_xml_import(tools.file_open(opj(m,xml)).read(), idref)
			elif tools.config["update"].has_key(m) or tools.config["update"].has_key("all"):
				for xml in update.get(m,[]):
					logger.notifyChannel("init", netsvc.LOG_INFO, 'addon:%s:update %s' % (m,xml))
					tools.convert_xml_import(tools.file_open(opj(m,xml)).read(), idref)
			del unloaded[m]
			loaded[m] = 1

==> tinyerp-3.0.1.tar.gz: 3.0.1/server/bin/addons/__init__.py <==
##############################################################################
#
# Copyright (c) 2004 TINY SPRL. (http://tiny.be)
#
# $Id: __init__.py 1308 2005-09-08 18:02:01Z pinky $
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contact a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import os, sys, imp

import tools

import netsvc
logger = netsvc.Logger()

opj = os.path.join
ad = tools.config['addons_path']
sys.path.insert(1,ad)

# create the list of modules to load
loaded={}
unloaded={}
init={}
demo={}
update={}
#CHECKME: je vois pas l'interet de dico 'active' vu qu'on vire de ttes facons les non actifs juste apres...
active={}
for i in os.listdir(ad):
	if os.path.isfile(opj(ad, i, "__terp__.py")):
		info = eval(file(opj(ad, i, "__terp__.py")).read())
		if info.get('active', True):
			unloaded[i] = info['depends']
			init[i] = info.get('init_xml', [])
			demo[i] = info.get('demo_xml', [])
			update[i] = info.get('update_xml', [])
		active[i]=info.get('active',True)

def load_files(m, file_dict, idref):
	for xml in file_dict.get(m, []):
		logger.notifyChannel("init", netsvc.LOG_INFO, 'addon:%s:loading %s' % (m,xml))
		tools.convert_xml_import(tools.file_open(opj(m,xml)).read(), idref)
		
# load modules
while len(unloaded):
	for (m,dep) in unloaded.items():
		ready = True
		for j in dep:
			if not (j in loaded or j in unloaded):
				logger.notifyChannel("init", netsvc.LOG_ERROR, "addon:%s:couldn't load module because of unmet dependency (%s) !" % (m,j))
				del unloaded[m]
			ready = ready and j in loaded
		if not active[m]:
			del unloaded[m]
			continue
		if ready:
			logger.notifyChannel("init", netsvc.LOG_INFO, 'addon:%s' % (m,))
			sys.stdout.flush()
			imp.load_module(m, *imp.find_module(m))
			idref = {}

			init_m = tools.config["init"].has_key(m) or tools.config["init"].has_key("all")
			demo_m = tools.config["demo"].has_key(m) or tools.config["demo"].has_key("all")
			update_m = tools.config["update"].has_key(m) or tools.config["update"].has_key("all")

			if init_m or demo_m:
				load_files(m, init, idref)
			if demo_m:
				load_files(m, demo, idref)
			if update_m or init_m:
				load_files(m, update, idref)
			
			del unloaded[m]
			loaded[m] = 1
